'''This is an intermediate script that call's Dan Kelson's cpgplot wrapper
module.  It's to make it look like Nick's wrappers, which pygplot was built
on and I don't feel like making changes to MY code.  Aside from some 
convenience functions, this wrapper mostly adds default arguments to the 
raw cpgplot calls.'''

from cpgplot import *
import numpy.oldnumeric as num

def get_num():
   return('numpy')

def histogram (a,numbins=10,defaultlimits=None,printextras=1):
   """ Returns (i) an array of histogram bin counts, (ii) the smallest value
   of the histogram binning, and (iii) the bin width (the last 2 are not
   necessarily integers).  Default number of bins is 10.  Defaultlimits
   can be None (the routine picks bins spanning all the numbers in the
   a) or a 2-sequence (lowerlimit, upperlimit).  Returns all of the
   following: array of bin values, lowerreallimit, binsize, extrapoints.

   Returns: (array of bin counts,bin-minimum,min-width, #-points-outside-range)
   """
   a = num.ravel(a)               # flatten any >1D arrays
   if (defaultlimits <> None):
      lowerreallimit = defaultlimits[0]
      upperreallimit = defaultlimits[1]
      binsize = (upperreallimit-lowerreallimit) / float(numbins)
   else:
      Min = num.minimum.reduce(a)
      Max = num.maximum.reduce(a)
      estbinwidth = float(Max - Min)/float(numbins) + 1
      binsize = (Max-Min+estbinwidth)/float(numbins)
      lowerreallimit = Min - binsize/2.0  #lower real limit,1st bin
   bins = num.zeros(numbins)
   extrapoints = 0
   for n in a:
      try:
         if (n-lowerreallimit) < 0:
            extrapoints = extrapoints + 1
         else:
            bintoincrement = int((n-lowerreallimit) / float(binsize))
            bins[bintoincrement] = bins[bintoincrement] + 1
      except:                           # point outside lower/upper limits
         extrapoints = extrapoints + 1
   if (extrapoints > 0 and printextras == 1):
      print '\nPoints outside given histogram range =',extrapoints
   return (bins, lowerreallimit, binsize, extrapoints)

def autocuts(array, percent):
   '''Given an array of image data, this function computes the lower and upper
   pixel values between which percent of the pixel values reside.  Useful
   for displaying image data.'''

   # First, bin the data into 2048 bins
   array = num.ravel(array)
   z2 = num.maximum.reduce(array)
   z1 = num.minimum.reduce(array)
   dz = (z2-z1)/2048
   bins,lowerreallimit,binsize,extrapoints = histogram(array, numbins=2048)

   npixels = array.shape[0]
   cutoff = int(npixels*(100-percent)/100.0)/2

   # Now we inch up either side of the histogram until we get cutoff
   #  pixels on either side
   npixels = 0
   for i in range(2048):
      nprev = npixels
      npixels += int(bins[i])
      if npixels >= cutoff:
         low = z1 + i*dz
         lower = low - dz
         if i != 0:
            interp = 1.0*(cutoff-nprev)/(npixels-nprev)
            low = lower + (low - lower)*interp
         break

   # Now for the upper limit
   npixels = 0
   for i in range(2047,-1,-1):
      nprev = npixels
      npixels += int(bins[i])
      if npixels >= cutoff:
         high = z1 + i*dz
         higher = high + dz
         if i != 2047:
            interp = 1.0*(cutoff-nprev)/(npixels-nprev)
            high = higher + (higher-high)*interp
         break
   return(high,low)

pgarro = cpgarro
pgask = cpgask

def pgaxis(x1,y1,x2,y2,opt="N",step=0.0,nsub=0,dmajl=0.5,dmajr=0.5,fmin=0.5,
      disp=0.3,orient=0.0):
   return cpgaxis(opt, x1, y1, x2, y2, v1, v2, step, nsub, dmajl, dmajr, fmin,
                     disp, orient)

def pgband(mode, i=0, xref=0.0, yref=0.0, x=0, y=0):
   return cpgband(mode, i, xref, yref, x, y)[1:4]

pgbbuf = cpgbbuf

def pgbeg(device='/XWINDOWN', xnsub=1, ynsub=1):
   ret = cpgbeg(0,device,xnsub,ynsub)
   if ret != 1:
      raise IOError, "Failed to open plot device."
   else:
      return ret

def pgbin(nbin, ox, od, center=1):
   szx = len(ox);   szd = len(od)
   if szx > szd:  szx = szd
   return cpgbin(nbin, x, od, center)
   
def pgbox(xopt=None, xtick=0.0, nxsub=0,
          yopt=None, ytick=0.0, nysub=0):
   if xopt is None:
      xopt = "ABCGNTS"
      yopt = "ABCGNTS"
   return cpgbox(xopt,xtick,nxsub,yopt,ytick,nysub)
   

pgcirc = cpgcirc
pgclos = cpgclos

def pgconb(oa, cd, rd, c1, c2, r1, r2, oc, nc, tr, blank=0.0):
   # cpgconb expects a 1-D array.
   if len(oa.shape) > 1: a = oa.flat
   else:  a = oa
   return cpgconb(a, cd, rd, c1+1, c2+1, r1+1, r2+1, oc, nc, tr, blank)

def pgconf(oa, cd, rd, c1, c2, r1, r2, l1, l2, tr):
   if len(oa.shape) > 1: a = oa.flat
   else:  a = oa
   return cpgconf(a, cd, rd, c1+1, c2+1, r1+1, r2+1, l1, l2, tr)

def pgconl(oa, cd, rd, c1, c2, r1, r2, c, tr, label, intval, minint):
   if len(oa.shape) > 1: a = oa.flat
   else:  a = oa
   return cpgconl(a, cd, rd, c1+1, c2+1, r1+1, r2+1, c, tr, label, intval, minint)

def pgcons(oa, cd, rd, c1, c2, r1, r2, c, nc, tr):
   if len(oa.shape) > 1: a = oa.flat
   else:  a = oa
   return cpgcons(a, cd, rd, c1+1, c2+1, r1+1, r2+1, oc, nc, tr)

def pgcont(oa, cd, rd, c1, c2, r1, r2, c, nc, tr):
   if len(oa.shape) > 1: a = oa.flat
   else:  a = oa
   return cpgcont(a, cd, rd, c1+1, c2+1, r1+1, r2+1, c, nc, tr)

def pgctab(l,r,g,b, nc=None, contra=1, bright=0.5):
   if nc is None:
      nc = len(l)
   return cpgctab(l,r,g,b,nc,contra,bright)

def pgcurs(x=0, y=0):
   return cpgcurs(x,y)

pgdraw = cpgdraw

pgebuf = cpgebuf

pgend = cpgend

def pgenv(xn,xx,yn,yx,just=0,type=0):
   return cpgenv(xn,xx,yn,yx,just,type)

pgeras = cpgeras

def pgerr1(dir, x, y, e, t=1):
   return cpgerr1(dir,x,y,e, t)

def pgerrb(dir, x, y, e, t=1):
   return cpgerrb(dir,len(x),x,y,e, t)

def pgerrx(x,y,e,t=1):
   return cpgerrx(len(x), x, y, e, t)

def pgerry(x,y,e,t=1):
   return cpgerry(len(x), x, y, e, t)

pgetxt = cpgetxt

pggray = cpggray

pghi2d = cpghi2d

def pghist_s(od, nbin, pgflag=0, datamin=None, datamax=None):
   pgflat = 0
   if datamin is None:
      datamin = num.min(od)
   if datamax is None:
      datamax = num.max(od)
   npts = len(od)
   return cpghist(npts, od, datamin, datamax, nbin, pgflag)
   
pgiden = cpgiden

def pgimag(oa, cd, rd, c1, c2, r1, r2, fg, bg, ot):
   a = oa.flat
   return cpgimag(a, cd, rd, c1+1, c2+1, r1+1, r2+1, fg, bg, ot)

pglab = cpglab

pglcur = cpglcur

pgldev = cpgldev

def pglen(text, units=4):
   return cpglen(units, text)

def pgline(x,y):
   return cpgline(len(x),x,y)

pgmove = cpgmove

def pgmtxt(side, disp, coord, fjust, text):
   return cpgmtxt(side, disp, coord, fjust, text)

pgncur = cpgncur

pgnumb = cpgnumb

pgolin = cpgolin

def pgopen(z='/XWINDOW'):
   return cpgopen(z)

pgpage = cpgpage

def pgpanl(nx=1,ny=1):
   return cpgpanl(nx,ny)
   
def pgpap(width,aspect=1):
   return cpgpap(width,aspect)

pgpixl = cpgpixl

pgpnts = cpgpnts

pgpoly = cpgpoly

def pgpt(x,y,symbol=0):
   return cpgpt(len(x), x, y, symbol)

pgpt1 = cpgpt1

def pgptxt(x,y,angle,fjust, string):
   return cpgptxt(x,y,angle,fjust,string)

pgqah = cpgqah

pgqcf = cpgqcf

pgqch = cpgqch

pgqci = cpgqci

pgqcir = cpgqcir

pgqclp = cpgqclp

def pgqcol():
   return cpgqcol()

pgqcr = cpgqcr

def pgqcs(units = 4):
   return cpgqcs(units)

def pgqdt(i):
   typ,tlen,descr,dlen,inter = cpgqdt(i)
   return(typ, descr, inter)

pgqfs = cpgqfs

pgqhs = cpgqhs

pgqid = cpgqid

pgqinf = cpgqinf

pgqitf = cpgqitf

pgqls = cpgqls

pgqlw = cpgqlw

def pgqndt():
   return cpgqndt()

pgqpos = cpgqpos

pgqtbg = cpgqtbg

def pgqtxt(x,y,angle,fjust,string):
   xa,ya = cpgqtxt(x,y,angle,fjust,string)
   return ((xa[0],xa[1],xa[2],xa[3]),(ya[0],ya[1],ya[2],ya[2]))

pgqvp = cpgqvp

pgqvsz = cpgqvsz

def pgqwin():
   return cpgqwin()

def pgrect(x1,x2,y1,y2):
   return cpgrect(x1,x2,y1,y2)

pgrnd = cpgrnd

pgrnge = cpgrnge

pgsah = cpgsah

def pgsave():
   return cpgsave()

def pgscf(i=1):
   return cpgscf(i)

def pgsch(s=1):
   return cpgsch(s)

def pgscir(cir1=0,cir2=0):
   if cir1 == 0 and cir2 == 0:
      cir1,cir2 = pgqcol()
      if cir2 > 15:  cir1 = 16
      else:  cir1 = 1
   return cpgscir(cir1,cir2)

def pgsci(i = 1):
   return cpgsci(i)

pgsclp = cpgsclp

pgscr = cpgscr

pgscrl = cpgscrl

pgscrn = cpgscrn

def pgsfs(fs=1):
   return cpgsfs(fs)

pgshls = cpgshls

pgshs = cpgshs

pgsitf = cpgsitf

def pgslct(did=0):
   return cpgslct(did)

def pgsls(ls = 0):
   return cpgsls(ls)

def pgslw(lw = 1):
   return cpgslw(lw)

pgstbg = cpgstbg

def pgsubp(nxsub=1, nysub=1):
   return cpgsubp(nxsub,nysub)

def pgsvp(xleft=0.0, xright=1.0, ybot=0.0, ytop=1.0):
   return cpgsvp(xleft,xright,ybot,ytop)

def pgswin(x1=0, x2=1.0, y1=0.0, y2=1.0):
   return cpgswin(x1,x2,y1,y2)

def pgtbox(xopt="ABCGNTS", xtick=0.0, nxsub=0, yopt="ABCGNTS", ytick=0.0, 
      nysub=0):
   return cpgtbox(xopt,xtick,nxsub,yopt,ytick,nysub)

pgtext = cpgtext

pgtick = cpgtick

def pgunsa():
   return cpgunsa()

pgupdt = cpgupdt

pgvect = cpgvect

pgvsiz = cpgvsiz

pgvstd = cpgvstd

pgwedg = cpgwedg

pgwnad = cpgwnad
